#ifndef _MARCHING_CUBES_
#define _MARCHING_CUBES_

#include "vectors.h"

// ==================================================================================
// A helper class for the Marching Cubes algorithm

class GridValue {
public:
  GridValue() {}
  GridValue(int x, int y, int z, float v) :
    position((float)x,(float)y,(float)z), 
    value(v) {}
  Vec3f position;
  float value;
};

// ==================================================================================
class MarchingCubes {

public:
  MarchingCubes(int _nx, int _ny, int _nz) : nx(_nx), ny(_ny), nz(_nz) {
    values = new float[nx*ny*nz]; }
  ~MarchingCubes() { delete [] values; }

  float get(int x, int y, int z) const {
    assert (x >= 0 && x < nx);
    assert (y >= 0 && y < ny);
    assert (z >= 0 && z < nz);
    return values[z*nx*ny + y*nx + x];
  }

  void set(int x, int y, int z, float v) const {
    assert (x >= 0 && x < nx);
    assert (y >= 0 && y < ny);
    assert (z >= 0 && z < nz);
    values[z*nx*ny + y*nx + x] = v;
  }

  // =============
  // THE DRAW CODE
  void Paint(float isosurface) const;

private:

  // private helper functions
  void PaintTetra(const GridValue &a, const GridValue &b, const GridValue &c, const GridValue &d, float isosurface) const;
  void PaintTetraHelper4(const GridValue &a, const GridValue &b, const GridValue &c, const GridValue &d, float isosurface) const;
  void PaintTetraHelper3(const GridValue &a, const GridValue &b, const GridValue &c, const GridValue &d, float isosurface) const;
  void PaintTetraHelper2(const GridValue &a, const GridValue &b, const GridValue &c, const GridValue &d, float isosurface) const;
  void PaintTetraHelper1(const GridValue &a, const GridValue &b, const GridValue &c, const GridValue &d, float isosurface) const;
  void drawIfBoundary(const Vec3f &p1, const Vec3f &p2, const Vec3f &p3) const;

  // ==============
  // REPRESENTATION
  int nx, ny, nz;
  float* values;
};

// ==================================================================================

void insertNormal(const Vec3f &p1, const Vec3f &p2, const Vec3f &p3);
void drawTriangle(const Vec3f &p1, const Vec3f &p2, const Vec3f &p3);

#endif

